package com.luojigou.product.utils

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import androidx.core.app.ActivityCompat

internal class PermissionUtils(private val context: Context) {
    companion object {
        private val blePermissions: Array<String> by lazy {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.S) {
                arrayOf(
                    Manifest.permission.BLUETOOTH_SCAN,
                    Manifest.permission.BLUETOOTH_ADVERTISE,
                    Manifest.permission.BLUETOOTH_CONNECT
                )
            } else {
                arrayOf(
                    Manifest.permission.ACCESS_FINE_LOCATION
                )
            }
        }

        private val wifiPermissions: Array<String> by lazy {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
                arrayOf(
                    Manifest.permission.ACCESS_FINE_LOCATION, Manifest.permission.CHANGE_WIFI_STATE
                )
            } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                arrayOf(Manifest.permission.CHANGE_WIFI_STATE)
            } else {
                arrayOf()
            }
        }

        private val notificationPermissions: Array<String> by lazy {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                arrayOf(Manifest.permission.POST_NOTIFICATIONS)
            } else {
                arrayOf()
            }
        }

        private val permissions: Array<String> by lazy {
            blePermissions + wifiPermissions + notificationPermissions
        }
    }

    fun checkAllPermissions(): Boolean {
        return permissions.find {
            ActivityCompat.checkSelfPermission(context, it) != PackageManager.PERMISSION_GRANTED
        } == null
    }

    fun shouldShowRequestPermissionRationale(): Boolean {
        return permissions.find {
            !ActivityCompat.shouldShowRequestPermissionRationale(context as Activity, it)
        } != null
    }

    fun requestAllPermissions(): Array<String> {
        return permissions.filter {
            ActivityCompat.checkSelfPermission(context, it) != PackageManager.PERMISSION_GRANTED
        }.toTypedArray()
    }
}