package com.luojigou.product.wifi

import android.net.wifi.ScanResult
import android.os.Build
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.luojigou.product.R

class XDScanWifiAdapter(
    private val itemClickListener: (ScanResult?) -> Unit,
) : RecyclerView.Adapter<XDScanWifiAdapter.ViewHolder>() {
    private val list: MutableList<ScanResult> = ArrayList()

    override fun getItemViewType(position: Int): Int =
        if (position == list.size) 1 else 0


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        if (viewType == 1) {
            val itemView: View = LayoutInflater.from(parent.context)
                .inflate(R.layout.item_wifi_add, parent, false)
            return ViewHolder(itemView)
        } else {
            val itemView: View = LayoutInflater.from(parent.context)
                .inflate(R.layout.item_wifi, parent, false)
            return ViewHolder(itemView)
        }
    }

    override fun getItemCount(): Int =
        list.size + 1

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        if (position == list.size) {
            holder.itemView.setOnClickListener {
                itemClickListener.invoke(null)
            }
        } else {
            val item = list[position]

            holder.name.text = item.ssid()

            if (item.capabilities.contains("WPA")) {
                holder.img.setImageResource(
                    when (item.level) {
                        in 0 downTo -30 -> R.drawable.wifi_lock4
                        in -31 downTo -67 -> R.drawable.wifi_lock3
                        in -68 downTo -79 -> R.drawable.wifi_lock2
                        else -> R.drawable.wifi_lock1
                    }
                )
            } else {
                holder.img.setImageResource(
                    when (item.level) {
                        in 0 downTo -30 -> R.drawable.wifi_level4
                        in -31 downTo -67 -> R.drawable.wifi_level3
                        in -68 downTo -79 -> R.drawable.wifi_level2
                        else -> R.drawable.wifi_level1
                    }
                )
            }

            holder.itemView.setOnClickListener {
                itemClickListener.invoke(item)
            }
        }
    }

    fun addValue(result: ScanResult) {
        val ssid = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            result.wifiSsid.toString()
        } else {
            result.BSSID
        }

        if (ssid != null && result.frequency in 2400..2500) {
            list.add(result)
            notifyItemChanged(list.size - 1)
        }
    }

    class ViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val img = itemView.findViewById<ImageView>(R.id.iv_wifi_rssi)
        val name = itemView.findViewById<TextView>(R.id.tv_wifi_name)
    }
}