package com.luojigou.product.xiaodou

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.net.wifi.ScanResult
import android.os.Bundle
import androidx.core.content.ContextCompat
import com.luojigou.product.R
import com.luojigou.product.base.BaseActivity
import com.luojigou.product.wifi.ssid
import com.luojigou.product.xiaodou.connect.XDConnectConnectingFragment
import com.luojigou.product.xiaodou.connect.XDConnectFailFragment
import com.luojigou.product.xiaodou.connect.XDConnectPasswordFragment
import com.luojigou.product.xiaodou.connect.XDConnectPasswordSsidFragment
import com.luojigou.product.xiaodou.connect.XDConnectSuccessFragment

class XDConnectActivity : BaseActivity() {
    companion object {
        const val ACTION_SELECT_WIFI_SSID = "com.luojigou.xiaodou.ble.ACTION_SELECT_WIFI_SSID"
        const val ACTION_SELECT_WIFI_WPA = "com.luojigou.xiaodou.ble.ACTION_SELECT_WIFI_WPA"

        fun startConnect(context: Context, wifi: ScanResult?) {
            val intent = Intent(context, XDConnectActivity::class.java)
            intent.putExtra(ACTION_SELECT_WIFI_SSID, wifi?.ssid())
            intent.putExtra(ACTION_SELECT_WIFI_WPA, wifi?.capabilities?.contains("WPA"))
            context.startActivity(intent)
        }
    }

    private val connectingFragment = XDConnectConnectingFragment()
    private val failFragment = XDConnectFailFragment()

    private val connectReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent) {
            setFragment(
                when (intent.action) {
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_TIMEOUT -> failFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_ERROR -> failFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_SUCCESS -> XDConnectSuccessFragment(
                        intent.getStringExtra("EXTRA_DATA")!!
                    )
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_CONNECTING -> connectingFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_FAIL -> failFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_NOT_FOUND -> failFragment
                    XDConnectService.ACTION_CONNECT_WIFI_RESULT_PASSWORD_ERROR -> failFragment
                    else -> failFragment
                }
            )
        }
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setTitle("连接Wi-Fi") //        setFragment(connectingFragment)

        val intentFilter = IntentFilter()
        intentFilter.apply {
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_TIMEOUT)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_ERROR)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_SUCCESS)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_CONNECTING)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_FAIL)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_NOT_FOUND)
            addAction(XDConnectService.ACTION_CONNECT_WIFI_RESULT_PASSWORD_ERROR)
        }

        ContextCompat.registerReceiver(
            this,
            connectReceiver,
            intentFilter,
            ContextCompat.RECEIVER_EXPORTED,
        )

        val ssid = intent.getStringExtra(ACTION_SELECT_WIFI_SSID)
        val wpa = intent.getBooleanExtra(ACTION_SELECT_WIFI_WPA, true)

        if (ssid != null) {
            setFragment(XDConnectPasswordFragment(ssid))
        } else {
            setFragment(XDConnectPasswordSsidFragment())
        }
    }

    override fun getLayoutId(): Int = R.layout.activity_connect

    override fun onDestroy() {
        unregisterReceiver(connectReceiver)
        super.onDestroy()
    }
}